package chess;

import chess.filters.JwtFilter;
import chess.engine.EngineHandler;
import chess.server.ServerStatus;
import chess.services.EngineInfoService;
import chess.services.UserInfoService;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.boot.context.embedded.FilterRegistrationBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;

import java.util.Arrays;


/**
 * This class contains base configuration for application.
 */
@EnableAutoConfiguration
@ComponentScan
@Configuration
@SuppressWarnings("unused")
public class AppConfig {

    @Bean
    public ServerStatus serverStatus() {
        return new ServerStatus();
    }

    @Bean
    public EngineHandler engineHandler() {
        return new EngineHandler();
    }

    @Bean
    public EngineInfoService engineInfoService() {
        return new EngineInfoService(Constants.ENGINES_CONFIG_PATH);
    }

    @Bean
    public UserInfoService userInfoService() {
        return new UserInfoService(Constants.USERS_FILE_PATH);
    }

    /**
     * This method defines which routes should be filtered by {@link JwtFilter}
     * @return configured FilterRegistrationBean
     */
    @Bean
    public FilterRegistrationBean jwtFilter() {
        FilterRegistrationBean registrationBean = new FilterRegistrationBean();
        registrationBean.setFilter(new JwtFilter(serverStatus()));
        registrationBean.setUrlPatterns(Arrays.asList(
                "/engine/*",
                "/ws_engine"
                ));
        return registrationBean;
    }

}
