package chess.api.controllers;

import chess.api.models.InfoListModel;
import chess.api.models.InfoModel;
import chess.api.models.StartEngineModel;
import chess.engine.EngineHandler;
import chess.services.EngineInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Collection;

/**
 * Handles all engine related routes
 */
@SuppressWarnings("unused")
@RestController
@RequestMapping("/engine")
public class EngineController {

    @Autowired
    private EngineHandler engineHandler;

    @Autowired
    private EngineInfoService engineInfoService;

    /**
     * @return list of available engines.
     */
    @RequestMapping("/available")
    public InfoListModel getAvailable() {
        Collection<EngineInfoService.EngineModel> engines = engineInfoService.getAvailableEngines();
        EngineInfoService.EngineModel[] enginesArray = engines.toArray(new EngineInfoService.EngineModel[engines.size()]);
        return new InfoListModel<>(true, enginesArray);
    }


    /**
     * Starts engine with name {@link StartEngineModel#engine}
     * @param startEngineModel object containing engine name user wants to start.
     * @return information whether engine was started.
     */
    @RequestMapping(value = "/start", method = RequestMethod.POST)
    public InfoModel start(@RequestBody StartEngineModel startEngineModel) {
        try {
            engineHandler.startEngine(engineInfoService.getEngineInfo(startEngineModel.engine));
        } catch (Exception e) {
            return new InfoModel(false, "Can`t start engine " + startEngineModel.engine);
        }
        return new InfoModel(true, String.format("Engine %s started", startEngineModel.engine));
    }

    /**
     * This method stops currently running engine.
     * @return whether the engine was stopped.
     */
    @RequestMapping(value = "/stop", method = RequestMethod.POST)
    public InfoModel stop() {
        engineHandler.stopEngine();
        return new InfoModel(true, "engine stoped");
    }
}
