package chess.api.controllers;

import chess.Constants;
import chess.api.models.AccessModel;
import chess.api.models.LoginModel;
import chess.server.ServerStatus;
import chess.services.UserInfoService;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.DigestUtils;
import org.springframework.web.bind.annotation.*;

import java.util.Date;

/**
 * This controller is responsible for handling all routes connected with user management.
 */
@RestController
@RequestMapping("/user")
@SuppressWarnings("unused")
public class UserController {

    @Autowired
    private ServerStatus serverStatus;

    @Autowired
    private UserInfoService userInfoService;

    /**
     * This method handles user login. First it checks whether user exists in {@link #userInfoService}. If user exists
     * this method will generate JWT token and return {@link AccessModel} object containing this token and information
     * about successful login. If user doesn't exists this method will return {@link AccessModel} without token and
     * with information about failed login.
     *
     * @param login object containing user login and password.
     * @return login result
     */
    @RequestMapping(path = "/login", method = RequestMethod.POST)
    public AccessModel getAccess(
            @RequestBody LoginModel login) {

        if (!userInfoService.loginUser(login)) {
            return new AccessModel(false, null);
        }

        String jwt = Jwts.builder()
                .setSubject(login.login)
                .claim("roles", "user")
                .setSubject(login.login)
                .setIssuedAt(new Date())
                .signWith(SignatureAlgorithm.HS256, Constants.ANDROID_APP_SECRET_KEY)
                .compact();

        return new AccessModel(true, jwt);
    }


    /**
     * This method logs out user
     */
    @RequestMapping(path = "/logout")
    public void leave() {
        serverStatus.userLoggedOut();
    }

}
