package chess.filters;

import chess.Constants;
import chess.server.ServerStatus;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;
import org.junit.Assert;
import org.junit.Test;
import org.mockito.Mockito;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import java.util.Date;

public class JwtFilterTest {

    private ServerStatus status = new ServerStatus();
    private JwtFilter filter = new JwtFilter(status);

    @Test
    public void rejectRequestWithoutAuthHeader() throws Exception {
        HttpServletRequest req = Mockito.mock(HttpServletRequest.class);
        Mockito.when(req.getHeader("authorization")).then(invocation -> null);

        ServletResponse res = Mockito.mock(ServletResponse.class);
        FilterChain chain = Mockito.mock(FilterChain.class);

        try {
            filter.doFilter(req, res, chain);
        } catch (ServletException e) {
            Assert.assertEquals(JwtFilter.ERROR_MISSING_OR_INVALID_HEADER, e.getMessage());
        }
    }

    @Test
    public void acceptCorrectRequest() throws Exception {
        String jwt = Jwts.builder()
                .claim("roles", "user")
                .setSubject("test")
                .setIssuedAt(new Date())
                .signWith(SignatureAlgorithm.HS256, Constants.ANDROID_APP_SECRET_KEY)
                .compact();

        HttpServletRequest req = Mockito.mock(HttpServletRequest.class);
        Mockito.when(req.getHeader("authorization")).then(invocation -> "Bearer " + jwt);
        ServletResponse res = Mockito.mock(ServletResponse.class);
        FilterChain chain = Mockito.mock(FilterChain.class);

        filter.doFilter(req, res, chain);

        Mockito.verify(chain, Mockito.times(1)).doFilter(req, res);
    }

}